// C Header File
// Created 4/2/2002; 3:57:57 PM
//CrashLib by Gregory Dietsche Copyright (C) 2002.

#ifndef __CRASH_H__
#define __CRASH_H__

#ifdef NO_EXIT_SUPPORT
	#error CrashLib requires Exit Support to function correctly
#endif

//version.h
//Crashlib needs TIGCC 0.94 beta 18 to function properly
#if __TIGCC__==0 && (__TIGCC_MINOR__< 94 && __TIGCC_BETA__ < 18)
	#warning Old TIGCC Version -- Crashlib may not work properly
#endif

/* #defines's */
#define VECT_ADDR(n)((void**)((n)*4))
#define VECT_ADDR_sh(s)((short)VECT_ADDR(s))
#define Crash_No_HUD NULL
#define NUM_VECT_REDIR (15)
#define NUM_VECT_SAVED ((NUM_VECT_REDIR)+6)

//If  you want to use the crash protection in a 'custom' manner
//then study this #define
#define AntiCrashProtection(HUD) \
	ERROR_FRAME __Crash_ERRFrame__; \
	unsigned short ___CrashERRCode__; \
	__gCrashHandler=HUD; \
	if((___CrashERRCode__=ER_catch(__Crash_ERRFrame__))) \
	{ \
		if(___CrashERRCode__==0x6) \
			___CrashERRCode__=0x0; \
		exit(___CrashERRCode__); \
	} \
	BackupExceptionVectors(__gVectors, __gOldVectors, NUM_VECT_SAVED); \
	InstallExceptionVectors(__gVectors, __gRedirect, NUM_VECT_REDIR); \
	atexit((atexit_t)atexit_RemoveAntiCrashProtection);

	
/* data structures */
typedef struct
{
	unsigned long  a7;
	unsigned long  d0;
	unsigned long  d1;
	unsigned long  d2;
	unsigned long  d3;
	unsigned long  d4;
	unsigned long  d5;
	unsigned long  d6;
	unsigned long  d7;
	unsigned long  a0;
	unsigned long  a1;
	unsigned long  a2;
	unsigned long  a3;
	unsigned long  a4;
	unsigned long  a5;
	unsigned long  a6;
	const unsigned char frametype;
	unsigned char  crashtype;
	
	union {
		
		struct {
			unsigned short sr;					/* STATUS REGISTER */
			unsigned long  pc;					/* PROGRAM COUNTER */
			}type_else;									/* Error Frame Types 1 and 2 */
		
		struct {
			unsigned short flags;				/* only bits 0-4 are used */
			unsigned long  address;			/* ACCESS ADDRESS */
			unsigned short ir;					/* INSTRUCTION REGISTER */
			unsigned short sr;					/* STATUS REGISTER */
			unsigned long  pc;					/* PROGRAM COUNTER */
			}type_zero;									/* Error Frame Type 0 */
			
		}frame;
		
	}CRASH_RECOVERY_STATE;

/* Function Prototypes - Public */
__ATTR_LIB_ASM__ void InstallExceptionVectors(const short vlist[], const long vnew[], short num);
__ATTR_LIB_ASM__ void BackupExceptionVectors(const short vlist[], long vsaved[], short num);

__ATTR_LIB_ASM__ void Crash_File_HUD(register CRASH_RECOVERY_STATE *dat asm("a0"));
__ATTR_LIB_ASM__ void Crash_Debug_HUD(register CRASH_RECOVERY_STATE *dat asm("a0"));
__ATTR_LIB_ASM__ void Crash_Simple_HUD(void);

/* Function Prototypes - Private */
void atexit_RemoveAntiCrashProtection(void);

/* Public Global Variables */
extern const char * const gCrashLibText[NUM_VECT_REDIR];
extern const char * const CrashLibVersion;
extern const char * const CrashLibBuildDate;
extern const char * const CrashLibBuildTime;

/* Private Global Variables */
extern void * __gCrashHandler;
extern long 	__gOldVectors[NUM_VECT_SAVED];
extern long 	__gRedirect[NUM_VECT_REDIR];
extern short 	__gVectors[NUM_VECT_SAVED];

#endif
